/*
 * MiniGame.cpp
 */

#define	F_CPU	12000000

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/eeprom.h>
#include	<avr/pgmspace.h>
#include	<util/delay.h>

#include	"sound.h"
#include	"graph.h"
#include	"button.h"

volatile uint8_t	RefreshCounter = 0;

/******************************************************************************
	Game Functions 
******************************************************************************/
extern uint8_t tetrisTitle(void);
extern uint8_t tetrisMain(void);
extern uint8_t tetrisOver(void);

extern uint8_t shootingMain(void);
extern uint8_t shootingTitle(void);
extern uint8_t shootingOver(void);

extern uint8_t jumpMain(void);
extern uint8_t jumpTitle(void);
extern uint8_t jumpOver(void);

typedef uint8_t (*FUNCPTR)(void);

typedef struct {
	const char *name;
	FUNCPTR	title;
	FUNCPTR	main;
	FUNCPTR	over;
} GAME_FUNC;

GAME_FUNC	GameFunc[] = {
	{ "TETRIS", tetrisTitle, tetrisMain, tetrisOver },
	{ "SHOOTING", shootingTitle, shootingMain, shootingOver },
	{ "JUMP", jumpTitle, jumpMain, jumpOver },
	{ NULL, NULL, NULL, NULL }
};

int8_t menu(void)
{
	static uint8_t cursor = 0;

	uint8_t i = 0;
	while( GameFunc[i].title != NULL ){
		fbDrawStr( 16, i*8, GameFunc[i].name, DRAW_MODE_PUT );
		i++;
	}
	if(i==0){
		fbDrawStr( 16, 0, "NO TITLE", DRAW_MODE_PUT );
		return 0;
	}

	uint8_t prevCursor = cursor;
	fbDrawChar( 8, cursor*8, ' ', DRAW_MODE_PUT );
	if( (btnOnFlag & BTN_UP   ) && cursor > 0  ) cursor--;	
	if( (btnOnFlag & BTN_DOWN ) && cursor < i-1) cursor++;
	if( prevCursor != cursor ) soundPlay( SOUND_TYPE_KEY );
	fbDrawChar( 8, cursor*8,     '>', DRAW_MODE_PUT );

	if( btnOnFlag & BTN_OK ){
		soundPlay( SOUND_TYPE_ENTER );
		fbClear();
		return cursor;
	}
	return -1;
}

/******************************************************************************
	Initialize I/O 
******************************************************************************/
void ioInit(void)
{
	// Global Pull Up Enable for All Input Pins
	MCUCR &= ~0x10;		

	// PORTC ... For Key Input with Pull-up
	DDRC = 0b00000000;
	PORTC= 0b11111111;

	// PORTD ... Not Used
	DDRD = 0b11111111;
	PORTD= 0b00000000;

	// PORTB ... SPI for LCD with SS, RS, RESET
	DDRB = 0b00101111; // Set to Output CLK, MOSI, SS, PB1(RS), PB0(RESET)
	PORTB= 0b11111111;
}

/******************************************************************************
	Timer1 for Interrupt
******************************************************************************/
#define TIMER_FREQ	60	//[Hz]
void timer1Init(void)
{
	// TCCR1A ... COM1A1 COM1A0 COM1B1 COM1B0 -     -    WGM11 WGM10
	// TCCR1B ... ICNC1  ICES1  -      WGM13  WGM12 CS12 CS11  CS10
	// COM1A[1:0] = 00		// Normal Operation
	// COM1B[1:0] = 00		// Normal Operation
	// ICNC1      = 0		// Input Capture Noise Canceler
	// ICES1      = 0		// Input Capture Edge Select
	// WGM1[3:0]  = 0100	// CTC mode with top value OCR1A
	// CS0[2:0]   = 010		// Clock Select = Clock/8

    cli();	// Disable Interrupt

	TCCR1A = 0b00000000;
	TCCR1B = 0b00001010;
	TCCR1C = 0;

	OCR1AH = ( ((F_CPU/(TIMER_FREQ*8))-1)>>8 ) & 0x00FF;
	OCR1AL = ( ((F_CPU/(TIMER_FREQ*8))-1)    ) & 0x00FF;

	OCR1BH = 0;		// Not Used
	OCR1BL = 0;		// Not Used
	
	TIMSK1 = 0b00000010;	// Compare A Match Interrupt Enable

    sei();	// Enable Interrupt
}

ISR(TIMER1_COMPA_vect)
{
	static uint8_t counter = 0;

	soundEngine();
	counter++;
	if(counter == 2){
		counter = 0;
		buttonManager();
		RefreshCounter++;
	}
}

/******************************************************************************
	Main Loop
******************************************************************************/
int main(void)
{
	uint8_t rc;
	uint8_t flag;
	uint8_t gameState = 0;
	uint8_t	prevRefreshCounter = 0;
	int8_t	gameNum = 0;

	ioInit();
	timer1Init();
	lcdInit();
	soundInit();

    while(1){
		flag = 1;
		while( RefreshCounter == prevRefreshCounter ) flag = 0;
		prevRefreshCounter = RefreshCounter;

		switch(gameState){
			case 0:
				gameNum = menu();
				if( gameNum >= 0 ) gameState = 1;
				break;
			case 1:
				rc = GameFunc[ gameNum ].title();
				if( rc != 0 ) gameState = 2;
				break;
			case 2:
				rc = GameFunc[ gameNum ].main();
				if( rc != 0 ) gameState = 3;
				break;
			case 3:
				rc = GameFunc[ gameNum ].over();
				if( rc != 0 ) gameState = 1;
				break;
		}

		if( flag != 0 ) fbDrawChar( 15, 0, '!', DRAW_MODE_PUT);
		lcdRefresh();
    }
}

